@@ -192,20 +192,17 @@ gst_rmdemux_get_type (void)
   return rmdemux_type;
 }
 
-GST_ELEMENT_REGISTER_DEFINE (rmdemux, "rmdemux",
-    GST_RANK_PRIMARY, GST_TYPE_RMDEMUX);
-
 static void
 gst_rmdemux_base_init (GstRMDemuxClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rmdemux_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rmdemux_videosrc_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rmdemux_audiosrc_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rmdemux_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rmdemux_videosrc_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rmdemux_audiosrc_template));
   gst_element_class_set_static_metadata (element_class, "RealMedia Demuxer",
       "Codec/Demuxer",
       "Demultiplex a RealMedia file into audio and video streams",
@@ -271,7 +268,6 @@ gst_rmdemux_init (GstRMDemux * rmdemux)
   rmdemux->have_group_id = FALSE;
   rmdemux->group_id = G_MAXUINT;
   rmdemux->flowcombiner = gst_flow_combiner_new ();
-  rmdemux->seek_seqnum = GST_SEQNUM_INVALID;
 
   gst_rm_utils_run_tests ();
 }
@@ -306,18 +302,8 @@ gst_rmdemux_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
     case GST_EVENT_SEEK:
     {
       gboolean running;
-      guint32 seqnum;
 
       GST_LOG_OBJECT (rmdemux, "Event on src: SEEK");
-
-      seqnum = gst_event_get_seqnum (event);
-      if (seqnum == rmdemux->seek_seqnum) {
-        GST_LOG_OBJECT (pad,
-            "Drop duplicated SEEK event seqnum %" G_GUINT32_FORMAT, seqnum);
-        gst_event_unref (event);
-        break;
-      }
-
       /* can't seek if we are not seekable, FIXME could pass the
        * seek query upstream after converting it to bytes using
        * the average bitrate of the stream. */
@@ -493,13 +479,10 @@ gst_rmdemux_perform_seek (GstRMDemux * rmdemux, GstEvent * event)
   GstSeekType cur_type, stop_type;
   gint64 cur, stop;
   gboolean update;
-  guint32 seqnum = GST_SEQNUM_INVALID;
-  GstEvent *fl_event;
 
   if (event) {
     GST_DEBUG_OBJECT (rmdemux, "seek with event");
 
-    seqnum = gst_event_get_seqnum (event);
     gst_event_parse_seek (event, &rate, &format, &flags,
         &cur_type, &cur, &stop_type, &stop);
 
@@ -528,15 +511,8 @@ gst_rmdemux_perform_seek (GstRMDemux * rmdemux, GstEvent * event)
   /* first step is to unlock the streaming thread if it is
    * blocked in a chain call, we do this by starting the flush. */
   if (flush) {
-    fl_event = gst_event_new_flush_start ();
-    if (seqnum != GST_SEQNUM_INVALID)
-      gst_event_set_seqnum (fl_event, seqnum);
-    gst_pad_push_event (rmdemux->sinkpad, fl_event);
-
-    fl_event = gst_event_new_flush_start ();
-    if (seqnum != GST_SEQNUM_INVALID)
-      gst_event_set_seqnum (fl_event, seqnum);
-    gst_rmdemux_send_event (rmdemux, fl_event);
+    gst_pad_push_event (rmdemux->sinkpad, gst_event_new_flush_start ());
+    gst_rmdemux_send_event (rmdemux, gst_event_new_flush_start ());
   } else {
     gst_pad_pause_task (rmdemux->sinkpad);
   }
@@ -551,13 +527,8 @@ gst_rmdemux_perform_seek (GstRMDemux * rmdemux, GstEvent * event)
   GST_LOG_OBJECT (rmdemux, "Took streamlock");
 
   if (event) {
-    if (!gst_segment_do_seek (&rmdemux->segment, rate, format, flags,
-            cur_type, cur, stop_type, stop, &update)) {
-      ret = FALSE;
-      goto done;
-    }
-
-    rmdemux->seek_seqnum = seqnum;
+    gst_segment_do_seek (&rmdemux->segment, rate, format, flags,
+        cur_type, cur, stop_type, stop, &update);
   }
 
   GST_DEBUG_OBJECT (rmdemux, "segment positions set to %" GST_TIME_FORMAT "-%"
@@ -566,10 +537,7 @@ gst_rmdemux_perform_seek (GstRMDemux * rmdemux, GstEvent * event)
 
   /* we need to stop flushing on the sinkpad as we're going to use it
    * next. We can do this as we have the STREAM lock now. */
-  fl_event = gst_event_new_flush_stop (TRUE);
-  if (seqnum != GST_SEQNUM_INVALID)
-    gst_event_set_seqnum (fl_event, seqnum);
-  gst_pad_push_event (rmdemux->sinkpad, fl_event);
+  gst_pad_push_event (rmdemux->sinkpad, gst_event_new_flush_stop (TRUE));
 
   GST_LOG_OBJECT (rmdemux, "Pushed FLUSH_STOP event");
 
@@ -607,24 +575,17 @@ gst_rmdemux_perform_seek (GstRMDemux * rmdemux, GstEvent * event)
     /* Reset the demuxer state */
     rmdemux->state = RMDEMUX_STATE_DATA_PACKET;
 
-    if (flush) {
-      fl_event = gst_event_new_flush_stop (TRUE);
-      if (seqnum != GST_SEQNUM_INVALID)
-        gst_event_set_seqnum (fl_event, seqnum);
-      gst_rmdemux_send_event (rmdemux, fl_event);
-    }
+    if (flush)
+      gst_rmdemux_send_event (rmdemux, gst_event_new_flush_stop (TRUE));
 
     /* must send newsegment event from streaming thread, so just set flag */
     rmdemux->need_newsegment = TRUE;
 
     /* notify start of new segment */
     if (rmdemux->segment.flags & GST_SEEK_FLAG_SEGMENT) {
-      GstMessage *msg =
+      gst_element_post_message (GST_ELEMENT_CAST (rmdemux),
           gst_message_new_segment_start (GST_OBJECT_CAST (rmdemux),
-          GST_FORMAT_TIME, rmdemux->segment.position);
-      if (seqnum != GST_SEQNUM_INVALID)
-        gst_message_set_seqnum (msg, seqnum);
-      gst_element_post_message (GST_ELEMENT_CAST (rmdemux), msg);
+              GST_FORMAT_TIME, rmdemux->segment.position));
     }
 
     /* restart our task since it might have been stopped when we did the 
@@ -767,8 +728,6 @@ gst_rmdemux_reset (GstRMDemux * rmdemux)
 
   rmdemux->have_group_id = FALSE;
   rmdemux->group_id = G_MAXUINT;
-
-  rmdemux->seek_seqnum = GST_SEQNUM_INVALID;
 }
 
 static GstStateChangeReturn
@@ -1008,7 +967,8 @@ need_pause:
         gst_rmdemux_send_event (rmdemux, gst_event_new_eos ());
       }
     } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_EOS) {
-      GST_ELEMENT_FLOW_ERROR (rmdemux, ret);
+      GST_ELEMENT_ERROR (rmdemux, STREAM, FAILED,
+          (NULL), ("stream stopped, reason %s", reason));
       gst_rmdemux_send_event (rmdemux, gst_event_new_eos ());
     }
     return;
@@ -2007,20 +1967,14 @@ gst_rmdemux_descramble_audio (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
   guint packet_size = stream->packet_size;
   guint height = stream->subpackets->len;
   guint leaf_size = stream->leaf_size;
-  guint size;
   guint p, x;
 
   g_assert (stream->height == height);
 
-  GST_LOG_OBJECT (rmdemux, "packet_size = %u, leaf_size = %u, height= %u",
-      packet_size, leaf_size, height);
+  GST_LOG ("packet_size = %u, leaf_size = %u, height= %u", packet_size,
+      leaf_size, height);
 
-  if (!g_uint_checked_mul (&size, height, packet_size)) {
-    GST_ERROR_OBJECT (rmdemux, "overflowing audio packet size");
-    return GST_FLOW_ERROR;
-  }
-
-  outbuf = gst_buffer_new_and_alloc (size);
+  outbuf = gst_buffer_new_and_alloc (height * packet_size);
   gst_buffer_map (outbuf, &outmap, GST_MAP_WRITE);
 
   for (p = 0; p < height; ++p) {
@@ -2150,20 +2104,14 @@ gst_rmdemux_descramble_sipr_audio (GstRMDemux * rmdemux,
   GstMapInfo outmap;
   guint packet_size = stream->packet_size;
   guint height = stream->subpackets->len;
-  guint size;
   guint p;
 
   g_assert (stream->height == height);
 
-  GST_LOG_OBJECT (rmdemux, "packet_size = %u, leaf_size = %u, height= %u",
-      packet_size, stream->leaf_size, height);
+  GST_LOG ("packet_size = %u, leaf_size = %u, height= %u", packet_size,
+      stream->leaf_size, height);
 
-  if (!g_uint_checked_mul (&size, height, packet_size)) {
-    GST_ERROR_OBJECT (rmdemux, "overflowing SIPR audio packet size");
-    return GST_FLOW_ERROR;
-  }
-
-  outbuf = gst_buffer_new_and_alloc (size);
+  outbuf = gst_buffer_new_and_alloc (height * packet_size);
   gst_buffer_map (outbuf, &outmap, GST_MAP_WRITE);
 
   for (p = 0; p < height; ++p) {
@@ -2174,13 +2122,7 @@ gst_rmdemux_descramble_sipr_audio (GstRMDemux * rmdemux,
       GST_BUFFER_PTS (outbuf) = GST_BUFFER_PTS (b);
     }
 
-    if (gst_buffer_extract (b, 0, outmap.data + packet_size * p,
-            packet_size) != packet_size) {
-      GST_ERROR_OBJECT (rmdemux, "not enough SIPR audio data available");
-      gst_buffer_unmap (outbuf, &outmap);
-      gst_buffer_unref (outbuf);
-      return GST_FLOW_ERROR;
-    }
+    gst_buffer_extract (b, 0, outmap.data + packet_size * p, packet_size);
   }
   gst_buffer_unmap (outbuf, &outmap);
 
@@ -2279,9 +2221,6 @@ gst_rmdemux_parse_video_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
 
   gst_buffer_map (in, &map, GST_MAP_READ);
 
-  if (map.size < offset)
-    goto not_enough_data;
-
   data = map.data + offset;
   size = map.size - offset;
 
@@ -2348,10 +2287,6 @@ gst_rmdemux_parse_video_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
     }
     GST_DEBUG_OBJECT (rmdemux, "fragment size %d", fragment_size);
 
-    if (fragment_size > map.size
-        || (data - map.data) > map.size - fragment_size)
-      goto not_enough_data;
-
     /* get the fragment */
     fragment =
         gst_buffer_copy_region (in, GST_BUFFER_COPY_ALL, data - map.data,
@@ -2370,17 +2305,15 @@ gst_rmdemux_parse_video_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
       stream->frag_length = fragment_size;
     }
 
-    if (stream->frag_count >= MAX_FRAGS) {
-      gst_buffer_unref (fragment);
-      goto too_many_fragments;
-    }
-
     /* put fragment in adapter */
     gst_adapter_push (stream->adapter, fragment);
     stream->frag_offset[stream->frag_count] = stream->frag_current;
     stream->frag_current += fragment_size;
     stream->frag_count++;
 
+    if (stream->frag_count > MAX_FRAGS)
+      goto too_many_fragments;
+
     GST_DEBUG_OBJECT (rmdemux, "stored fragment in adapter %d/%d",
         stream->frag_current, stream->frag_length);
 
@@ -2502,9 +2435,6 @@ gst_rmdemux_parse_audio_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
   GstFlowReturn ret;
   GstBuffer *buffer;
 
-  if (gst_buffer_get_size (in) < offset)
-    goto not_enough_data;
-
   buffer = gst_buffer_copy_region (in, GST_BUFFER_COPY_MEMORY, offset, -1);
 
   if (rmdemux->first_ts != -1 && timestamp > rmdemux->first_ts)
@@ -2535,19 +2465,9 @@ gst_rmdemux_parse_audio_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
     ret = gst_pad_push (stream->pad, buffer);
   }
 
-done:
   gst_buffer_unref (in);
 
   return ret;
-
-  /* ERRORS */
-not_enough_data:
-  {
-    GST_ELEMENT_WARNING (rmdemux, STREAM, DECODE, ("Skipping bad packet."),
-        (NULL));
-    ret = GST_FLOW_OK;
-    goto done;
-  }
 }
 
 static GstFlowReturn
@@ -2568,9 +2488,6 @@ gst_rmdemux_parse_packet (GstRMDemux * rmdemux, GstBuffer * in, guint16 version)
   data = map.data;
   size = map.size;
 
-  if (size < 2 + 4 + 2)
-    goto not_enough_data;
-
   /* stream number */
   id = RMDEMUX_GUINT16_GET (data);
 
@@ -2606,8 +2523,6 @@ gst_rmdemux_parse_packet (GstRMDemux * rmdemux, GstBuffer * in, guint16 version)
 
   /* version 1 has an extra byte */
   if (version == 1) {
-    if (size < 1)
-      goto not_enough_data;
     data += 1;
     size -= 1;
   }
@@ -2679,14 +2594,11 @@ unknown_stream:
     gst_buffer_unref (in);
     return GST_FLOW_OK;
   }
+}
 
-  /* ERRORS */
-not_enough_data:
-  {
-    GST_ELEMENT_WARNING (rmdemux, STREAM, DECODE, ("Skipping bad packet."),
-        (NULL));
-    gst_buffer_unmap (in, &map);
-    gst_buffer_unref (in);
-    return GST_FLOW_OK;
-  }
+gboolean
+gst_rmdemux_plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "rmdemux",
+      GST_RANK_PRIMARY, GST_TYPE_RMDEMUX);
 }
